###########################################################
## Replicate table of errors on boundary bias simulation ##
###########################################################

# DGP 
mu = function(x){ log(1 + exp(6 * x[1])) }

# function to run a simulation 
simulation.run = function(n, p, sigma, ntest = 1000){
  errors = replicate(num.reps, {
    X = matrix(runif(n * p, -1, 1), nrow = n)
    Y = apply(X, MARGIN = 1, FUN = mu) + sigma * rnorm(n)

    X.test = matrix(runif(ntest*p, 0, 1), nrow = ntest)
    truth = apply(X.test, MARGIN = 1, FUN = mu)
    
    # train llf 
    ll.forest = ll_regression_forest(X, Y, honesty = TRUE, enable.ll.split = TRUE, tune.parameters = tune)

    # use lasso to select linear correction variables
    lasso.mod = cv.glmnet(X, Y, alpha = 1)
    lasso.coef = predict(lasso.mod, type = "nonzero") 
    # if lasso chose no variables, use all for LL correction
    if(!is.null(dim(lasso.coef))){
      selected = as.numeric(lasso.coef[,1])
    } else {
      selected = 1:ncol(X)
    }
    llf.preds = predict(ll.forest, X.test, linear.correction.variables = selected, ll.lambda = ll.lambda,
                        ll.weight.penalty = TRUE)$predictions
    llf.mse = mean((llf.preds - truth)**2)

    forest = regression_forest(X, Y, honesty = TRUE, tune.parameters = tune)
    rf.preds = predict(forest, X.test)$predictions
    rf.mse = mean((rf.preds - truth)**2)

    inds = sample(n, size=n/2, replace = FALSE)
    lasso.mod = cv.glmnet(X[inds,], Y[inds], alpha = 1)
    lasso.preds = predict(lasso.mod, newx = X[-inds,], s ="lambda.min")
    lasso.resids = as.numeric(Y[-inds] - lasso.preds)

    rf = regression_forest(X[-inds,], lasso.resids, honesty = TRUE)
    lasso.rf.preds = predict(rf, newdata = X.test)$predictions + predict(lasso.mod, newx = X.test, s = "lambda.min")
    lasso.rf.mse = mean((lasso.rf.preds - truth)**2)

    bart.mod = wbart(X, Y, X.test, ndpost = ndpost)
    bart.preds = bart.mod$yhat.test.mean
    bart.mse = mean((bart.preds - truth)**2)

    boost.cv.fit = rlearner::cvboost(as.matrix(X), Y, ntrees_max = xgb_max, num_search_rounds = num_search_rounds)
    xgb.preds = predict(boost.cv.fit, as.matrix(X.test))
    xg.mse = mean((xgb.preds - truth)**2)

    c(llf.mse, rf.mse, lasso.rf.mse, bart.mse, xg.mse)
  })
  c("LLF" = mean(errors[1,]),
    "RF"= mean(errors[2,]),
    "Lasso-RF" = mean(errors[3,]),
    "BART" = mean(errors[4,]),
    "XGBoost" = mean(errors[5,]))
}

# arguments for table; customize to examine behavior on other data settings
ps = c(5, 50)
ns = c(1000, 5000)
sigmas = c(0.1, 2)

if(efficient.run){
  # restrict to lower-data example
  ns = 1000
}

# iterate through arguments and find RMSE 
args = expand.grid(n = ns, p = ps, sigma = sigmas)
full.results = t(apply(args, MARGIN = 1, FUN = function(arguments){
  print(arguments)
  mses = simulation.run(arguments[1], arguments[2], arguments[3])
  sqrt(mses)
}))
full.results = cbind(args, round(full.results, 3))

# store results 
write.csv(full.results, "results/boundary_bias.csv", row.names = FALSE)
