#####################################
## Errors on the Friedman function ##
#####################################

ff = function(x){
  return(10*sin(pi*x[1]*x[2]) + 20*((x[3] - 0.5)**2) + 10*x[4] + 5*x[5])
}

simulation.run = function(n, p, sigma, ntest = 1000){
  errors = replicate(num.reps, {
    X = matrix(runif(n * p, 0, 1), nrow = n)
    Y = apply(X, MARGIN = 1, FUN = ff) + sigma * rnorm(n)

    X.test = matrix(runif(ntest * p, 0, 1), nrow = ntest)
    truth = apply(X.test, MARGIN = 1, FUN = ff)

    forest = regression_forest(X, Y, honesty = TRUE, tune.parameters = tune)
    rf.preds = predict(forest, X.test)$predictions
    rf.mse = mean((rf.preds - truth)**2)
    
    forest = ll_regression_forest(X, Y, honesty = TRUE, tune.parameters = tune)
    
    # use lasso to select linear correction variables
    lasso.mod = cv.glmnet(X, Y, alpha = 1)
    lasso.coef = predict(lasso.mod, type = "nonzero") 
    # if lasso chose no variables, use all for LL correction
    if(!is.null(dim(lasso.coef))){
      selected = as.numeric(lasso.coef[,1])
    } else {
      selected = 1:ncol(X)
    }
    llf.preds = predict(forest, X.test, linear.correction.variables = selected, ll.lambda = ll.lambda)$predictions
    llf.mse = mean((llf.preds - truth)**2)

    inds = sample(n, size = n/2, replace = FALSE)
    lasso.mod = cv.glmnet(X[inds,], Y[inds], alpha = 1)
    lasso.preds = predict(lasso.mod, newx = X[-inds,], s = "lambda.min")
    lasso.resids = as.numeric(Y[-inds] - lasso.preds)

    rf = regression_forest(X[-inds,], lasso.resids, honesty = TRUE)
    lasso.rf.preds = predict(rf, newdata = X.test)$predictions + predict(lasso.mod, newx = X.test, s = "lambda.min")
    lasso.rf.mse = mean((lasso.rf.preds-truth)**2)

    bart.mod = wbart(X, Y, X.test, ndpost = ndpost)
    bart.preds = bart.mod$yhat.test.mean
    bart.mse = mean((bart.preds - truth)**2)

    boost.cv.fit = rlearner::cvboost(as.matrix(X), Y, ntrees_max = xgb_max, num_search_rounds = num_search_rounds)
    xgb.preds = predict(boost.cv.fit, as.matrix(X.test))
    xg.mse = mean((xgb.preds - truth)**2)

    c(llf.mse, rf.mse, lasso.rf.mse, bart.mse, xg.mse)
  })
  c("LLF" = mean(errors[1,]),
    "RF" = mean(errors[2,]),
    "Lasso-RF" = mean(errors[3,]),
    "BART" = mean(errors[4,]),
    "XGBoost" = mean(errors[5,]))
}

ps = seq(10, 50, by = 20)
ns = c(1000, 5000)
sigmas = c(5, 20)

if(efficient.run){
  # restrict to smaller sample size
  ns = 1000
  
  # restrict to low-dimensional example
  ps = c(10, 30)
}

# time for 1 run
ptm = proc.time()

args = expand.grid(n = ns, p = ps, sigma = sigmas)
full.results = t(apply(args, MARGIN = 1, FUN = function(arguments){
  print(arguments)
  mses = simulation.run(arguments[1], arguments[2], arguments[3])
  sqrt(mses)
}))
full.results = cbind(args, round(full.results, 3))

full.results
cat(paste("One run took", (1/60) * round(proc.time() - ptm)[3]), " minutes")

write.csv(full.results, "results/friedman_table.csv", row.names = FALSE)
