# load the data
data = read.csv("data/cps2018.csv")

# add age^2, educ^2 covariates
data$agesq = data$age**2
data$educsq = data$educ**2

covariates = c("agesq", "educsq", "uhrswork1", "famsize", "occ2010", "occ10ly", "sex", "race",
               "marst", "labforce", "ind1950", "classwkr", "wkstat", "metro")

continuous.covariates = which(covariates %in% c("agesq", "educsq", "uhrswork1", "famsize"))
outcome = "incwage"

data = data[,c(covariates, outcome)]
data = data[complete.cases(data),]

# transform outcome (standard for wage regressions)
data$incwage = log(data$incwage + 1)

######################
## ERROR EVALUATION ##
######################

size.test = 40000
sample.sizes = c(2000, 5000, 10000, 50000)

if(efficient.run){
  sample.sizes = sample.sizes = c(2000, 5000, 10000)
}

ptm = proc.time()

mse.sample.sizes = data.frame(t(sapply(sample.sizes, function(size){
  index.train = sample(1:nrow(data), size = size, replace = FALSE)

  X = data[index.train, covariates]
  Y = data$incwage[index.train]

  X = X[complete.cases(X),]
  Y = Y[complete.cases(X)]

  results = data.frame(t(sapply(1:num.reps, function(i){
    print(i)

    index.test = sample((1:nrow(data))[-index.train], size = size.test, replace = FALSE)

    X.test = data[index.test, covariates]
    truth = data$incwage[index.test]

    X.test = X.test[complete.cases(X.test),]
    truth = truth[complete.cases(X.test)]

    llforest = ll_regression_forest(as.matrix(X), Y, honesty = TRUE, tune.parameters = tune)

    # use lasso to select linear correction variables
    lasso.mod = cv.glmnet(as.matrix(X[,continuous.covariates]), Y, alpha = 1)
    lasso.coef = predict(lasso.mod, type = "nonzero") 
    # if lasso chose no variables, use all for LL correction
    if(!is.null(dim(lasso.coef))){
      selected = as.numeric(lasso.coef[,1])
    } else {
      selected = continuous.covariates
    }
    
    llf.preds = predict(llforest, as.matrix(X.test),
                        linear.correction.variables = selected,
                        ll.lambda = ll.lambda,
                        ll.weight.penalty = TRUE)$predictions
    llf.mse = mean((llf.preds - truth)**2)

    forest = regression_forest(as.matrix(X), Y, honesty = TRUE, tune.parameters = tune)
    rf.preds = predict(forest, as.matrix(X.test))$predictions
    rf.mse = mean((rf.preds - truth)**2)

    ols.form = as.formula(paste("Y", paste(covariates, collapse = "+"), sep = "~"))
    dd.ols = cbind(Y, X)
    ols.fit = lm(ols.form, dd.ols)
    ols.preds = predict(ols.fit, X.test)
    ols.mse = mean((ols.preds - truth)**2)

    mm = model.matrix( ~.^2, data = X)
    lasso.mod = cv.glmnet(mm, Y, alpha = 1)
    mmtest = model.matrix( ~.^2, data = X.test)
    lasso.preds = predict(lasso.mod, newx = mmtest, lambda= lasso.mod$lambda.min)
    lasso.mse = mean((lasso.preds - truth)**2)

    bart.mod = wbart(X, Y, X.test, ndpost = ndpost)
    bart.preds = bart.mod$yhat.test.mean
    bart.mse = mean((bart.preds-truth)**2)

    boost.cv.fit = rlearner::cvboost(as.matrix(X), Y, ntrees_max = xgb_max, num_search_rounds = num_search_rounds)
    xgb.preds = predict(boost.cv.fit, as.matrix(X.test))
    xg.mse = mean((xgb.preds - truth)**2)

    return(c(ols.mse, llf.mse, rf.mse, lasso.mse, xg.mse, bart.mse))
  })))

  mses = colMeans(results)
  sds = apply(results, MARGIN = 2, FUN = sd)

  as.numeric(c(mses, sds))
})))

colnames(mse.sample.sizes) = c("OLS", "LLF", "RF", "Lasso", "XG", "BART",
                               "OLS.sd", "LLF.sd", "RF.sd", "Lasso.sd", "XG.sd", "BART.sd")

mse.sample.sizes$size = sample.sizes

cat(paste("One run took", (1/60) * round(proc.time() - ptm)[3]), " minutes")

write.csv(mse.sample.sizes,"results/wages_sample_sizes.csv", row.names = FALSE)
